@@ -1,3 +1,4 @@
+import logging
 import sys
 from collections import namedtuple, defaultdict
 from pathlib import Path
@@ -16,8 +17,8 @@ from weaveio.hierarchy import Multiple, unwind, collect, Hierarchy, find_branch
 from weaveio.opr3.hierarchy import APS, OB, OBSpec, Exposure, WeaveTarget, _predicate, Run, ArmConfig, FibreTarget, Fibre
 from weaveio.opr3.l1 import L1Spectrum, L1SingleSpectrum, L1StackSpectrum, L1SupertargetSpectrum
 from weaveio.opr3.l2 import L2, L2Single, L2Stack, L2Superstack, L2Supertarget, IngestedSpectrum, Fit, ModelSpectrum, Redrock, \
-    RVSpecfit, Ferre, PPXF, Gandalf, GandalfModelSpectrum, CombinedIngestedSpectrum, CombinedModelSpectrum, Template, RedshiftArray, IvarIngestedSpectrum, IvarCombinedIngestedSpectrum, MaskedCombinedIngestedSpectrum, GandalfEmissionModelSpectrum, GandalfCleanModelSpectrum, \
-    GandalfCleanIngestedSpectrum, L2Product, gandalf_line_names, gandalf_index_names
+    RVSpecfit, Ferre, PPXF, Gandalf, GandalfModelSpectrum, CombinedIngestedSpectrum, CombinedModelSpectrum, Template, RedshiftArray, GandalfEmissionModelSpectrum, GandalfCleanModelSpectrum, \
+    GandalfCleanIngestedSpectrum, L2Product, gandalf_line_names, gandalf_index_names, UncombinedIngestedSpectrum, UncombinedModelSpectrum, BaseCombinedModelSpectrum
 from weaveio.opr3.l1files import L1File, L1SuperstackFile, L1StackFile, L1SingleFile, L1SupertargetFile
 from weaveio.writequery import CypherData, CypherVariable
 from weaveio.writequery.actions import string_append
@@ -58,30 +59,38 @@ def extract_lines_names(expected_line_names, colnames):
     """
     returns dictionary of required factor names and values that can be take from the hdu
     """
-    actual_line_names = sorted([i[len('FLUX') + 1:] for i in colnames if i.startswith('FLUX')], key=lambda x: float(x.split('_')[1]))
+    colnames = list(map(lambda x: x.lower(), colnames))
+    expected_line_names = list(map(lambda x: x.lower(), expected_line_names))
+    actual_line_names = sorted([i[len('FLUX') + 1:] for i in colnames if i.startswith('flux')], key=lambda x: float(x.split('_')[1]))
     d = {}
     for name in actual_line_names:
         if name in expected_line_names:
-            name = name.lower()
             d[f"{name}_aon"] = f'aon_{name}'
             d[f"{name}_ebmv"] = f'ebv_{name}'
             d[f"{name}_flux"] = f'flux_{name}'
-            d[f"{name}_flux_error"] = f'err_flux_{name}'
-            d[f"{name}_amp_error"] = f'err_ampl_{name}'
+            d[f"{name}_flux_error"] = f'flux_err_{name}'
+            d[f"{name}_amp_error"] = f'ampl_err_{name}'
             d[f"{name}_redshift"] = f'z_{name}'
-            d[f"{name}_redshift_error"] = f'err_z_{name}'
+            d[f"{name}_redshift_error"] = f'z_err_{name}'
             d[f"{name}_sigma"] = f'sigma_{name}'
-            d[f"{name}_sigma_error"] = f'err_sigma_{name}'
+            d[f"{name}_sigma_error"] = f'sigma_err_{name}'
+        else:
+            logging.warning(f"line `{name}` was found in file but is not part of the schema (will be ignored)")
+    for name in expected_line_names:
+        if name not in actual_line_names:
+            logging.warning(f"expected line `{name}` not found in l2file[gandalf] hdu")
     return d
 
 
 def extract_indices_names(expected_index_names, colnames):
     d = {}
     for name in expected_index_names:
+        name = name.lower()
         if name in colnames:
-            name = name.lower()
             d[f'{name}'] = name
             d[f'{name}_error'] = f'err_{name}'
+        else:
+            logging.warning(f"index `{name}` was not found in l2file[gandalf] hdu")
     return d
 
 FitSpecs = namedtuple('FitSpecs', ['individuals', 'individual_models', 'combined', 'combined_model', 'colour_codes', 'nrow'])
@@ -95,9 +104,10 @@ class L2File(File):
     antimatch_pattern = '.*cube.*'
     L2 = L2Product
     L1 = L1Spectrum
-    parents = [Multiple(L1File, 2, 3), APS, Multiple(L2, maxnumber=1000)]
-    children = []
+    parents = [Multiple(L1File, 2, 3), Multiple(L2, maxnumber=1000, one2one=True)]
+    children = [APS]
     parts = ['RR', 'RVS', 'FR', 'GAND', 'PPXF']
+    recommended_batchsize = 50
     hdus = {
         'primary': PrimaryHDU,
         'class_table': TableHDU,
@@ -112,13 +122,13 @@ class L2File(File):
     def length(cls, path, part=None):
         hdus = fits.open(path)
         if part is None:
-            return len(hdus[1].data)
+            return hdus[1].header['NAXIS2']
         d = {'RR': 1, 'RVS': 2, 'FR':2, 'GAND': 3, 'PPXF': 3}
-        return len(hdus[d[part]].data)
+        return hdus[d[part]].header['NAXIS2']
 
     @classmethod
-    def make_l2(cls, spectra, aps, nspec, **hiers):
-        return cls.L2(**{cls.L1.plural_name: [spectra[i] for i in range(nspec)], 'aps': aps}, **hiers)
+    def make_l2(cls, spectra, nspec, **hiers):
+        return cls.L2(**{cls.L1.plural_name: [spectra[i] for i in range(nspec)]}, **hiers)
 
     @classmethod
     def decide_filetype(cls, l1filetypes: List[Type[File]]) -> Type[File]:
@@ -224,7 +234,7 @@ class L2File(File):
 
     @classmethod
     def attach_products_to_spectra(cls, specs: FitSpecs, formatter, hdu, names: Dict[str, str], types, combined_suffix='_C'):
-        IngestedSpectrum, CombinedIngestedSpectrum, ModelSpectrum, CombinedModelSpectrum = types
+        UncombinedIngestedSpectrum, CombinedIngestedSpectrum, ModelSpectrum, CombinedModelSpectrum = types
         # this is less complicated than it looks
         # `specs` is a namedtuple containing ingested and model spectra collections ([fibre1_spectrum, fibre2_spectrum, ...])
         # in the case of `individual_models` and `individuals`, they are a collection of collections:
@@ -241,7 +251,7 @@ class L2File(File):
         if specs.individuals is not None:
             with unwind(specs.individuals, specs.colour_codes, specs.nrow) as (spectra, colour_codes, nrow):
                 with unwind(spectra, colour_codes) as (spectrum, colour_code):
-                    spectrum = IngestedSpectrum.from_cypher_variable(spectrum)
+                    spectrum = UncombinedIngestedSpectrum.from_cypher_variable(spectrum)
                     for product in spectrum.products:
                         column_name = string_append(f'{names.get(product, product)}_{formatter}_'.upper(), colour_code)
                         spectrum.attach_product(product, hdu, column_name=column_name, index=nrow)
@@ -278,18 +288,21 @@ class L2File(File):
         unrolled = [specs.individual_models[i] for i in range(nl1specs)]
         for template_name in Redrock.template_names:
             chi2s = row[f'CZZ_CHI2_{template_name}'.lower()]
-            template = Template(model_spectra=unrolled, combined_model_spectrum=specs.combined_model,
+            template = Template(l1_spectra=l1spectra, uncombined_model_spectra=unrolled, combined_model_spectrum=specs.combined_model,
                                 redshift_array=zs[template_name], name=template_name, chi2_array=chi2s)
             templates[template_name] = template
-        return Redrock(l1spectra=l1spectra, model_spectra=unrolled, combined_model_spectrum=specs.combined_model, tables=row, tables_replace=replacements, **templates)
+        return Redrock(l1_spectra=l1spectra, uncombined_model_spectra=unrolled, combined_model_spectrum=specs.combined_model, tables=row, tables_replace=replacements, **templates)
 
     @classmethod
     def make_gandalf_structure(cls, l1spectra, specs, row, this_fname, nrow, replacements):
         model, ingested = specs.combined_model, specs.combined
         emission = GandalfEmissionModelSpectrum(gandalf_model_spectrum=model)
         clean_model = GandalfCleanModelSpectrum(gandalf_model_spectrum=model)
-        clean_ingested = GandalfCleanIngestedSpectrum(combined_ingested_spectrum=ingested)
-        gandalf = Gandalf(l1spectra=l1spectra, gandalf_model_spectrum=specs.combined_model, tables=row, tables_replace=replacements)
+        clean_ingested = GandalfCleanIngestedSpectrum(gandalf_model_spectrum=model)
+        gandalf = Gandalf(l1_spectra=l1spectra, model=specs.combined_model,
+                          clean_model=clean_model, clean_observed=clean_ingested,
+                          emission_line_model=emission,
+                          tables=row, tables_replace=replacements)
         return gandalf, GandalfSpecs(model=model, ingested=ingested, emission=emission, clean_model=clean_model,
                                      clean_ingested=clean_ingested, nrow=specs.nrow)
 
@@ -299,21 +312,20 @@ class L2File(File):
                              IngestedSpectrumClass: Optional[Type[IngestedSpectrum]],
                              CombinedIngestedSpectrumClass: Optional[Type[CombinedIngestedSpectrum]],
                              ModelSpectrumClass: Optional[Type[ModelSpectrum]],
-                             CombinedModelSpectrumClass: Optional[Type[CombinedModelSpectrum]],
+                             CombinedModelSpectrumClass: Optional[Type[BaseCombinedModelSpectrum]],
                              uses_disjoint_spectra: bool,
                              uses_combined_spectrum: Optional[bool],
-                             formatter: str,
-                             aps):
+                             formatter: str):
         # if the joint spectrum is not available, we dont read it, obvs
         if uses_combined_spectrum is None:
             uses_combined_spectrum = any('_C' in i for i in spectrum_hdu.data.names)
         fibre = Fibre.find(id=row['aps_id'])
         with unwind(CypherData(parent_l1filenames)) as l1_fname:  # for each parent l1 file
             l1file = L1File.find(fname=l1_fname)
-            _, fibretarget, l1spectrum, _ = find_branch(fibre, FibreTarget, L1Spectrum, l1file)  # get the fibre target and l1 spectrum for this fibre and file
+            _, l1spectrum, fibretarget, _ = find_branch(l1file, L1Spectrum, FibreTarget, fibre)  # get the fibre target and l1 spectrum for this fibre and file
             colour_code = ArmConfig.find(anonymous_children=[l1spectrum])['colour_code']
             if uses_disjoint_spectra:
-                individual = IngestedSpectrumClass(l1_spectrum=l1spectrum, aps=aps)
+                individual = IngestedSpectrumClass(l1_spectrum=l1spectrum)
                 individual_model = ModelSpectrumClass(uncombined_ingested_spectrum=individual)
         # now collect spec and models relative to the fibretarget
         if uses_disjoint_spectra:
@@ -323,84 +335,84 @@ class L2File(File):
             l1files, l1spectra, fibretargets, colour_codes = collect(l1file, l1spectrum, fibretarget,  colour_code)
             individuals, individual_models = None, None
         if uses_combined_spectrum:
-            combined = CombinedIngestedSpectrumClass(l1_spectra=[l1spectra[i] for i in range(len(parent_l1filenames))], aps=aps)
+            combined = CombinedIngestedSpectrumClass(l1_spectra=[l1spectra[i] for i in range(len(parent_l1filenames))])
             combined_model = CombinedModelSpectrumClass(combined_ingested_spectrum=combined)
         else:
             combined, combined_model = None, None
         return FitSpecs(individuals, individual_models, combined, combined_model, colour_codes, nrow), l1spectra, fibretargets
 
     @classmethod
-    def read_redrock(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, aps, zs, **hiers):
+    def read_redrock(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, zs, **hiers):
         if len(spectrum_hdu.data) == 0:
             return
         replacements = column_name_acronym_replacements(colnames, 'rr')
         with unwind(safe_table, enumerated=True) as (row, nrow):
             specs, l1spectra, fibretargets = cls.read_l2product_table(this_fname, spectrum_hdu, row, nrow,
-                                                          parent_l1filenames, IvarIngestedSpectrum,
-                                                          IvarCombinedIngestedSpectrum,
-                                                          ModelSpectrum, CombinedModelSpectrum,
-                                                          True, None, 'RR', aps)
+                                                          parent_l1filenames, UncombinedIngestedSpectrum,
+                                                          CombinedIngestedSpectrum,
+                                                          UncombinedModelSpectrum, CombinedModelSpectrum,
+                                                          True, None, 'RR')
             redrock = cls.make_redrock_fit(l1spectra, specs, zs, row, len(parent_l1filenames), replacements)
-            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), aps=aps, fibre_target=fibretargets[0], **hiers)
+            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), fibre_target=fibretargets[0], **hiers)
             l2.attach_optionals(redrock=redrock)
         l2, redrocks, *r = collect(l2, redrock, *specs)
-        return l2, redrocks, FitSpecs(*r), IvarIngestedSpectrum, IvarCombinedIngestedSpectrum, ModelSpectrum, CombinedModelSpectrum
+        return l2, redrocks, FitSpecs(*r), UncombinedIngestedSpectrum, CombinedIngestedSpectrum, UncombinedModelSpectrum, CombinedModelSpectrum
 
     @classmethod
-    def read_rvspecfit(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, aps, **hiers):
+    def read_rvspecfit(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, **hiers):
         if len(spectrum_hdu.data) == 0:
             return
         replacements = column_name_acronym_replacements(colnames, 'rvs')
         with unwind(safe_table, enumerated=True) as (row, nrow):
             rvs_specs, l1spectra, fibretargets = cls.read_l2product_table(this_fname, spectrum_hdu, row, nrow,
-                                                      parent_l1filenames, IngestedSpectrum,
+                                                      parent_l1filenames, UncombinedIngestedSpectrum,
                                                       CombinedIngestedSpectrum,
-                                                      ModelSpectrum, CombinedModelSpectrum,
-                                                      True, None, 'RVS', aps)
-            rvspecfit = RVSpecfit(l1spectra=l1spectra, model_spectra=[rvs_specs.individual_models[i] for i in range(len(parent_l1filenames))],
+                                                      UncombinedModelSpectrum, CombinedModelSpectrum,
+                                                      True, None, 'RVS')
+            rvspecfit = RVSpecfit(l1_spectra=l1spectra, uncombined_model_spectra=[rvs_specs.individual_models[i] for i in range(len(parent_l1filenames))],
                                   combined_model_spectrum=rvs_specs.combined_model, tables=row, tables_replace=replacements)
-            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), aps=aps, fibre_target=fibretargets[0], **hiers)
+            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), fibre_target=fibretargets[0], **hiers)
             l2.attach_optionals(rvspecfit=rvspecfit)
         l2, rvspecfits, *r = collect(l2, rvspecfit, *rvs_specs)
-        return l2, rvspecfits, FitSpecs(*r), IngestedSpectrum, CombinedIngestedSpectrum, ModelSpectrum, CombinedModelSpectrum
+        return l2, rvspecfits, FitSpecs(*r), UncombinedIngestedSpectrum, CombinedIngestedSpectrum, UncombinedModelSpectrum, CombinedModelSpectrum
 
     @classmethod
-    def read_ferre(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, aps, **hiers):
+    def read_ferre(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, **hiers):
         if len(spectrum_hdu.data) == 0:
             return
         replacements = column_name_acronym_replacements(colnames, 'fr')
         with unwind(safe_table, enumerated=True) as (row, nrow):
             ferre_specs, l1spectra, fibretargets = cls.read_l2product_table(this_fname, spectrum_hdu, row, nrow,
-                                                        parent_l1filenames, IngestedSpectrum,
+                                                        parent_l1filenames, UncombinedIngestedSpectrum,
                                                         CombinedIngestedSpectrum,
-                                                        ModelSpectrum, CombinedModelSpectrum,
-                                                        True, None, 'FR', aps)
-            ferre = Ferre(l1spectra=l1spectra, model_spectra=[ferre_specs.individual_models[i] for i in range(len(parent_l1filenames))],
+                                                        UncombinedModelSpectrum, CombinedModelSpectrum,
+                                                        True, None, 'FR')
+            ferre = Ferre(l1_spectra=l1spectra, uncombined_model_spectra=[ferre_specs.individual_models[i] for i in range(len(parent_l1filenames))],
                           combined_model_spectrum=ferre_specs.combined_model, tables=row, tables_replace=replacements)
-            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), aps=aps, fibre_target=fibretargets[0], **hiers)
+            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), fibre_target=fibretargets[0], **hiers)
             l2.attach_optionals(ferre=ferre)
         l2, ferres, *r = collect(l2, ferre, *ferre_specs)
-        return l2, ferres, FitSpecs(*r), IngestedSpectrum, CombinedIngestedSpectrum, ModelSpectrum, CombinedModelSpectrum
+        return l2, ferres, FitSpecs(*r), UncombinedIngestedSpectrum, CombinedIngestedSpectrum, UncombinedModelSpectrum, CombinedModelSpectrum
 
     @classmethod
-    def read_ppxf(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, aps, **hiers):
+    def read_ppxf(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, **hiers):
         if len(spectrum_hdu.data) == 0:
             return
         replacements = column_name_acronym_replacements(colnames, 'ppxf')
         with unwind(safe_table, enumerated=True) as (row, nrow):
             ppxf_specs, l1spectra, fibretargets = cls.read_l2product_table(this_fname, spectrum_hdu, row, nrow,
                                                          parent_l1filenames, None,
-                                                         MaskedCombinedIngestedSpectrum,
+                                                         CombinedIngestedSpectrum,
                                                          None, CombinedModelSpectrum,
-                                                         False, True, 'PPXF', aps)
-            ppxf = PPXF(l1spectra=l1spectra, combined_model_spectrum=ppxf_specs.combined_model, tables=row, tables_replace=replacements)
-            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), aps=aps, fibre_target=fibretargets[0], **hiers)
+                                                         False, True, 'PPXF')
+            ppxf = PPXF(l1_spectra=l1spectra, combined_model_spectrum=ppxf_specs.combined_model, tables=row, tables_replace=replacements)
+            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), fibre_target=fibretargets[0], **hiers)
             l2.attach_optionals(ppxf=ppxf)
         l2, ppxfs, *r = collect(l2, ppxf, *ppxf_specs)
-        return l2, ppxfs, FitSpecs(*r), None, MaskedCombinedIngestedSpectrum, None, CombinedModelSpectrum
+        return l2, ppxfs, FitSpecs(*r), None, CombinedIngestedSpectrum, None, CombinedModelSpectrum
 
     @classmethod
-    def read_gandalf(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, aps, **hiers):
+    def read_gandalf(cls, this_fname, spectrum_hdu, colnames, safe_table: CypherVariable, parent_l1filenames, **hiers):
         if len(spectrum_hdu.data) == 0:
             return
         replacements = column_name_acronym_replacements(colnames, 'gand')
@@ -409,14 +421,14 @@ class L2File(File):
         with unwind(safe_table, enumerated=True) as (row, nrow):
             gandalf_specs, l1spectra, fibretargets = cls.read_l2product_table(this_fname, spectrum_hdu, row, nrow,
                                                                               parent_l1filenames, None,
-                                                                              MaskedCombinedIngestedSpectrum,
+                                                                              CombinedIngestedSpectrum,
                                                                               None, GandalfModelSpectrum,
-                                                                              False, True, 'GAND', aps)
+                                                                              False, True, 'GAND')
             gandalf, gandalf_extra_specs = cls.make_gandalf_structure(l1spectra, gandalf_specs, row, this_fname, nrow, replacements)
-            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), aps=aps, fibre_target=fibretargets[0], **hiers)
+            l2 = cls.make_l2(l1spectra, nspec=len(parent_l1filenames), fibre_target=fibretargets[0], **hiers)
             l2.attach_optionals(gandalf=gandalf)
         l2, gandalfs, *r = collect(l2, gandalf, *gandalf_specs, *gandalf_extra_specs)
-        return l2, gandalfs, FitSpecs(*r[:len(gandalf_specs)]), GandalfSpecs(*r[len(gandalf_specs):]), None, MaskedCombinedIngestedSpectrum, None, GandalfModelSpectrum
+        return l2, gandalfs, FitSpecs(*r[:len(gandalf_specs)]), GandalfSpecs(*r[len(gandalf_specs):]), None, CombinedIngestedSpectrum, None, GandalfModelSpectrum
 
     @classmethod
     def make_redshift_arrays(cls, tbl: Table):
@@ -452,25 +464,25 @@ class L2File(File):
             cols = [col for col in safe_tables[i].colnames if not ('chi2' not in col and 'czz_' in col)]
             safe_cypher_tables[i] =  CypherData(safe_tables[i][cols])
         if part == 'RVS':
-            l2, specfits, specs, *types = cls.read_rvspecfit(path, astropy_hdus[5], astropy_hdus[2].data.names,
-                                                 safe_cypher_tables[2], fnames, aps, **hierarchies)
+            l2, specfits, specs, *types = cls.read_rvspecfit(path, astropy_hdus[5], [x.lower() for x in astropy_hdus[2].data.names],
+                                                 safe_cypher_tables[2], fnames, **hierarchies)
             hdu_node = 5
         elif part == 'FR':
-            l2, specfits, specs, *types = cls.read_ferre(path, astropy_hdus[5], astropy_hdus[2].data.names,
-                                             safe_cypher_tables[2], fnames, aps, **hierarchies)
+            l2, specfits, specs, *types = cls.read_ferre(path, astropy_hdus[5], [x.lower() for x in astropy_hdus[2].data.names],
+                                             safe_cypher_tables[2], fnames, **hierarchies)
             hdu_node = 5
         elif part == 'PPXF':
-            l2, specfits, specs, *types = cls.read_ppxf(path, astropy_hdus[6], astropy_hdus[3].data.names,
-                                            safe_cypher_tables[3], fnames, aps, **hierarchies)
+            l2, specfits, specs, *types = cls.read_ppxf(path, astropy_hdus[6], [x.lower() for x in astropy_hdus[3].data.names],
+                                            safe_cypher_tables[3], fnames, **hierarchies)
             hdu_node = 6
         elif part == 'RR':
             zs = cls.make_redshift_arrays(safe_tables[1])
-            l2, specfits, specs, *types = cls.read_redrock(path, astropy_hdus[4], astropy_hdus[1].data.names,
-                                               safe_cypher_tables[1], fnames, aps, zs, **hierarchies)
+            l2, specfits, specs, *types = cls.read_redrock(path, astropy_hdus[4], [x.lower() for x in astropy_hdus[1].data.names],
+                                               safe_cypher_tables[1], fnames, zs, **hierarchies)
             hdu_node = 4
         elif part == 'GAND':
-            l2, specfits, specs, extra_specs, *types = cls.read_gandalf(path, astropy_hdus[6], astropy_hdus[3].data.names,
-                             safe_cypher_tables[3], fnames, aps, **hierarchies)
+            l2, specfits, specs, extra_specs, *types = cls.read_gandalf(path, astropy_hdus[6], [x.lower() for x in astropy_hdus[3].data.names],
+                             safe_cypher_tables[3], fnames, **hierarchies)
             hdu_node = 6
         else:
             raise ValueError(f"{part} is not a valid part")
@@ -486,8 +498,8 @@ class L2File(File):
 
 class L2SingleFile(L2File):
     singular_name = 'l2single_file'
-    children = []
-    parents = [Multiple(L1SingleFile, 2, 3, constrain=(Exposure,)), APS, Multiple(L2Single, maxnumber=1000)]
+    children = [APS]
+    parents = [Multiple(L1SingleFile, 2, 3, constrain=(Exposure,)), Multiple(L2Single, maxnumber=1000)]
     L2 = L2Single
     L1 = L1SingleSpectrum
 
@@ -502,8 +514,8 @@ class L2SingleFile(L2File):
 
 class L2StackFile(L2File):
     singular_name = 'l2stack_file'
-    children = []
-    parents = [Multiple(L1StackFile, 1, 3, constrain=(OB,)), APS, Multiple(L2Stack, maxnumber=1000)]
+    children = [APS]
+    parents = [Multiple(L1StackFile, 1, 3, constrain=(OB,)), Multiple(L2Stack, maxnumber=1000)]
     L2 = L2Stack
     L1 = L1StackSpectrum
 
@@ -515,11 +527,11 @@ class L2StackFile(L2File):
 
 class L2SuperstackFile(L2File):
     singular_name = 'l2superstack_file'
-    children = []
+    children = [APS]
     parents = [Multiple(L1SingleFile, 0, 3, constrain=(OBSpec,)),
                Multiple(L1StackFile, 0, 3, constrain=(OBSpec,)),
                Multiple(L1SuperstackFile, 0, 3, constrain=(OBSpec,)),
-               APS, Multiple(L2Superstack, maxnumber=1000)]
+               Multiple(L2Superstack, maxnumber=1000)]
     L2 = L2Superstack
     L1 = L1Spectrum
 
@@ -532,8 +544,8 @@ class L2SuperstackFile(L2File):
 class L2SupertargetFile(L2File):
     singular_name = 'l2supertarget_file'
     match_pattern = 'WVE_*aps.fits'
-    children = []
-    parents = [Multiple(L1SupertargetFile, 2, 3, constrain=(WeaveTarget,)), APS, L2Supertarget]
+    children = [APS]
+    parents = [Multiple(L1SupertargetFile, 2, 3, constrain=(WeaveTarget,)), L2Supertarget]
     L2 = L2Supertarget
     L1 = L1SupertargetSpectrum
 
